#!/usr/bin/env python
#*****************************************************************************#
#**
#**  Weighted Random Classifier
#**
#**    Kyle D. Feuz, 2014
#**
#** Tools by the Center for Advanced Studies in Adaptive Systems at
#**  the School of Electrical Engineering and Computer Science at
#**  Washington State University
#** 
#** Copyright Washington State University, 2014
#** Copyright Kyle D. Feuz, 2014
#** 
#** Licenced under the Creative Commons (http://creativecommons.org):
#** Attribution-NonCommercial-ShareAlike 3.0 Unported
#** 
#** If this code is used for public research, any resulting publications need
#** to cite work done by Kyle D. Feuz at the Center for Advanced Study of 
#** Adaptive Systems (CASAS) at Washington State University.
#** 
#** Contact: Kyle D. Feuz (kyle.feuz@wsu.edu)
#** Contact: Diane J. Cook (cook@eecs.wsu.edu)
#*****************************************************************************#

import Orange

        
class Learner(object):
    def __new__(cls, examples=None, **kwds):
        learner = object.__new__(cls, **kwds)
        if examples:
            learner.__init__(**kwds)
            return learner(examples)
        else:
            return learner

    def __init__(self, name='random', **kwds):
        self.__dict__.update(kwds)
        self.name = name


    def __call__(self, examples, weight=None, **kwds):
        for k in kwds.keys():
            self.__dict__[k] = kwds[k]

        # obtain class distribution
        c = [0] * len(examples.domain.classVar.values)
        for e in examples:
            c[int(e.getclass())] += 1
        rand = Orange.misc.Random(0)
        max_i = sum(c)
        return Classifier(domain=examples.domain, distribution=c, 
                          max_i=max_i, rand=rand)
        
        

class Classifier:
    def __init__(self, **kwds):
        self.__dict__.update(kwds)

    def __call__(self, example, result_type=Orange.classification.Classifier.GetValue):

        if not self.domain:
            self.domain = example.domain
        v = self.rand(self.max_i)
        total = 0
        for i,c in enumerate(self.distribution):
           total += c
           if v < total:
              cl = Orange.data.Value(self.domain.classVar, i)
              break

        if result_type == Orange.classification.Classifier.GetValue:
            return cl
        if result_type == Orange.classification.Classifier.GetProbabilities:
            return None
        return (cl,None)
